
Bot = class.new();

function Bot:constructor(config)
	if( not config or type(config) ~= "table" ) then
		error("Did not receive config table.", 2);
	end
	self.config = config;

	self.socket = nil;
	self.ready = false;
	self.password = nil;
	self.lastMsg = nil;
	self.status = nil;
end

function Bot:destructor()
	self.config = nil;
	self.socket = nil;
	self.ready = false;
	self.password = nil;
	self.lastMsg = nil;
	self.status = nil;

end

function Bot:command(cmd)
	if( not self.socket ) then
		error("Cannot send commands without a socket.", 2);
	end

	if( not cmd ) then
		error("Cannot send an empty command", 2);
	end

	self.socket:send(cmd .. "\r\n");
	print("Send> ", cmd);
end

function Bot:connect(host, port)
	if( not host ) then
		error("Host was not specified", 2);
	end
	port = port or 6667; -- Default port

	self.socket = network.socket("tcp");

	local success = self.socket:connect(host, port);
	if( success ) then
		self.status = "wait_til_ready";
		self.lastMsg = time.getNow();
		printf("Connected to %s. Must now wait until end of MOTD.\n", host);
	else
		printf("Failed to connect to %s\n", host);
	end
end

function Bot:run()
	if( self.status == "wait_til_ready" ) then
		if( time.diff(self.lastMsg) > 20 ) then
			self.ready = true;
			printf("NOTICE: Timed out; Did not reach end of MOTD. Assuming ready.\n");
		end

		if( self.ready ) then
			print("NOTICE:\t\tReady.");
			self.status = "ready";
			self:init();
		else
			return;
		end
	end
end

-- Now we're connected, so send PASS, NICK, USER, and JOIN messages.
function Bot:init()
	self:user(config.username, config.realname);
	self:pass(config.password);
	self:nick(config.username);
	self:join(config.channel);
end

function Bot:pass(password)
	password = password or string.random("alnum", 16);

	self.password = password;
	local cmd = string.format("PASS %s", self.password);
	self:command(cmd);
end

function Bot:nick(nick)
	if( not nick or type(nick) ~= "string" ) then
		error("nick() must receive a name.", 2);
	end

	local cmd = string.format("NICK %s", nick);
	self:command(cmd);
end

function Bot:user(username, realname)
	if( not bot.ready ) then
		error("Not ready.", 2);
	end

	local cmd = string.format("USER %s %s %s :%s", username, '0', '*', realname)
	self:command(cmd);
end

function Bot:join(channel)
	if( not channel or type(channel) ~= "string" ) then
		error("join() must receive a channel name.", 2);
	end

	local cmd = string.format("JOIN %s", channel);
	self:command(cmd);
end

function Bot:say(receiver, msg)
	if( not receiver or not msg ) then
		error("We need a receiver and message to say something...", 2);
	end

	local cmd = string.format("PRIVMSG %s :%s", receiver, msg);
	self:command(cmd);
end

function Bot:quit(msg)
	if( self.socket ) then
		local msg = msg or "Quit";

		self:command(string.format("QUIT :%s", msg));
		system.rest(500);
		self.socket:close();
	end

	self.socket = nil;
	self.ready = false;
end


function Bot:recv(...)
	--[[
		NOTE: We *must* wait until all data is received (ie. MOTD) from the server
		after we connect, before we can actually do anything else.
	]]

	local sockId, message = ...;

	-- Iterate over each received line.
	for line in string.gmatch(message, "[^\r\n]+") do
		-- Respond to end of MOTD msg
		if( not self.ready ) then
			local _,match = string.match(line, "^:([a-zA-Z0-9.-]+) %d+ (.*) :End of /MOTD command.");
			if( match ) then
				bot.ready = true;
				printf("-----------------------------------\n\n[DEBUG]\t\tMOTD signaled END\n");
			end
		end

		local _,notice = string.match(line, "^:([a-zA-Z0-9.-]+) NOTICE %* :(.*)");
		if( notice ) then
			printf("[NOTICE]\t%s\n", notice);
		end

		-- Respond to PING requests with a PONG
		local daemon = string.match(line, "^PING (.*)");
		if( daemon ) then
			printf("Received ping request; respond with pong.\n");
			local cmd = string.format("PONG %s", daemon);
			self:command(cmd);
		end

		-- Respond to errors
		local errmsg = string.match(line, "^ERROR :(.*)");
		if( errmsg ) then
			self.socket:close();
			self.socket = nil;

			error(sprintf("We had a horrible error :(\nError: %s", errmsg), 0);
		end

		local from,channel,msg = string.match(line, "^:([a-zA-Z0-9.-]+)!.*@.* PRIVMSG (.*) :(.*)");
		if( from and channel and msg ) then
			printf("PRIVMSG\t[%s]\t%s: \'%s\'\n", channel, from, msg);

			if( msg == "!test" ) then
				self:say(channel, "Test successful.");
			else
				printf("Msg did not match. DEBUG: \'%s\'\n", msg);
			end
		end
	end

	self.lastMsg = time.getNow();
end